@@ -4,7 +4,7 @@
     This file is part of Corrade.
 
     Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-                2017, 2018, 2019 Vladimír Vondruš <mosra@centrum.cz>
+                2017, 2018, 2019, 2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -43,14 +43,14 @@ namespace Corrade { namespace Utility {
 @brief String utilities
 
 This library is built if `WITH_UTILITY` is enabled when building Corrade. To
-use this library with CMake, you need to request the `Utility` component of the
-`Corrade` package and link to the `Corrade::Utility` target.
+use this library with CMake, request the `Utility` component of the `Corrade`
+package and link to the `Corrade::Utility` target.
 
 @code{.cmake}
 find_package(Corrade REQUIRED Utility)
 
 # ...
-target_link_libraries(your-app Corrade::Utility)
+target_link_libraries(your-app PRIVATE Corrade::Utility)
 @endcode
 
 See also @ref building-corrade and @ref corrade-cmake for more information.
@@ -67,6 +67,8 @@ namespace Implementation {
     CORRADE_UTILITY_EXPORT std::string trim(std::string string, Containers::ArrayView<const char> characters);
 
     CORRADE_UTILITY_EXPORT std::vector<std::string> splitWithoutEmptyParts(const std::string& string, Containers::ArrayView<const char> delimiters);
+    CORRADE_UTILITY_EXPORT std::string join(const std::vector<std::string>& strings, Containers::ArrayView<const char> delimiter);
+    CORRADE_UTILITY_EXPORT std::string joinWithoutEmptyParts(const std::vector<std::string>& strings, Containers::ArrayView<const char> delimiter);
 
     CORRADE_UTILITY_EXPORT bool beginsWith(Containers::ArrayView<const char> string, Containers::ArrayView<const char> prefix);
     CORRADE_UTILITY_EXPORT bool endsWith(Containers::ArrayView<const char> string, Containers::ArrayView<const char> suffix);
@@ -286,19 +288,87 @@ with @cpp " \t\f\v\r\n" @ce as second parameter.
 */
 CORRADE_UTILITY_EXPORT std::vector<std::string> splitWithoutEmptyParts(const std::string& string);
 
+/**
+@brief Partition a string
+@m_since{2019,10}
+
+Equivalent to Python's @m_class{m-doc-external} [str.partition()](https://docs.python.org/3/library/stdtypes.html#str.partition).
+Splits @p string at the first occurence of @p separator. First returned value
+is the part before the separator, second the separator, third a part after the
+separator. If the separator is not found, returns the input string followed by
+two empty strings.
+@see @ref rpartition(), @ref Directory::splitExtension()
+*/
+CORRADE_UTILITY_EXPORT Containers::StaticArray<3, std::string> partition(const std::string& string, char separator);
+
+/**
+@overload
+@m_since{2019,10}
+*/
+CORRADE_UTILITY_EXPORT Containers::StaticArray<3, std::string> partition(const std::string& string, const std::string& delimiter);
+
+/**
+@brief Right-partition a string
+@m_since{2019,10}
+
+Equivalent to Python's @m_class{m-doc-external} [str.rpartition()](https://docs.python.org/3/library/stdtypes.html#str.rpartition).
+Splits @p string at the last occurence of @p separator. First returned value is
+the part before the separator, second the separator, third a part after the
+separator. If the separator is not found, returns two empty strings followed by
+the input string.
+@see @ref partition(), @ref Directory::splitExtension()
+*/
+CORRADE_UTILITY_EXPORT Containers::StaticArray<3, std::string> rpartition(const std::string& string, char separator);
+
+/**
+@overload
+@m_since{2019,10}
+*/
+CORRADE_UTILITY_EXPORT Containers::StaticArray<3, std::string> rpartition(const std::string& string, const std::string& delimiter);
+
 /**
 @brief Join strings with given character
 @param strings      Strings to join
 @param delimiter    Delimiter
 */
-CORRADE_UTILITY_EXPORT std::string join(const std::vector<std::string>& strings, char delimiter);
+inline std::string join(const std::vector<std::string>& strings, char delimiter) {
+    return Implementation::join(strings, {&delimiter, 1});
+}
+
+/**
+@overload
+@m_since{2019,10}
+*/
+template<std::size_t size> inline std::string join(const std::vector<std::string>& strings, const char(&delimiter)[size]) {
+    return Implementation::join(strings, {delimiter, size - 1});
+}
+
+/**
+@overload
+@m_since{2019,10}
+*/
+inline std::string join(const std::vector<std::string>& strings, const std::string& delimiter) {
+    return Implementation::join(strings, {delimiter.data(), delimiter.size()});
+}
 
 /**
 @brief Join strings with given character and remove empty parts
 @param strings      Strings to join
 @param delimiter    Delimiter
 */
-CORRADE_UTILITY_EXPORT std::string joinWithoutEmptyParts(const std::vector<std::string>& strings, char delimiter);
+inline std::string joinWithoutEmptyParts(const std::vector<std::string>& strings, char delimiter) {
+    return Implementation::joinWithoutEmptyParts(strings, {&delimiter, 1});
+}
+
+/** @overload */
+template<std::size_t size> inline std::string joinWithoutEmptyParts(const std::vector<std::string>& strings, const char(&delimiter)[size]) {
+    return Implementation::joinWithoutEmptyParts(strings, {delimiter, size - 1});
+}
+
+/** @overload */
+inline std::string joinWithoutEmptyParts(const std::vector<std::string>& strings, const std::string& delimiter) {
+    return Implementation::joinWithoutEmptyParts(strings, {delimiter.data(), delimiter.size()});
+}
 
 /**
 @brief Convert string to lowercase
